@@ -2537,15 +2537,15 @@ out:
     return ret;
 }
 
-static int
-netdev_offload_dpdk_flow_flush(struct netdev *netdev)
+static void
+flush_netdev_flows_in_related(struct netdev *netdev, struct netdev *related)
 {
-    struct cmap *map = offload_data_map(netdev);
-    struct ufid_to_rte_flow_data *data;
     unsigned int tid = netdev_offload_thread_id();
+    struct cmap *map = offload_data_map(related);
+    struct ufid_to_rte_flow_data *data;
 
     if (!map) {
-        return -1;
+        return;
     }
 
     CMAP_FOR_EACH (data, node, map) {
@@ -2556,6 +2556,31 @@ netdev_offload_dpdk_flow_flush(struct netdev *netdev)
             netdev_offload_dpdk_flow_destroy(data);
         }
     }
+}
+
+static bool
+flush_in_vport_cb(struct netdev *vport,
+                  odp_port_t odp_port OVS_UNUSED,
+                  void *aux)
+{
+    struct netdev *netdev = aux;
+
+    /* Only vports are related to physical devices. */
+    if (netdev_vport_is_vport_class(vport->netdev_class)) {
+        flush_netdev_flows_in_related(netdev, vport);
+    }
+
+    return false;
+}
+
+static int
+netdev_offload_dpdk_flow_flush(struct netdev *netdev)
+{
+    flush_netdev_flows_in_related(netdev, netdev);
+
+    if (!netdev_vport_is_vport_class(netdev->netdev_class)) {
+        netdev_ports_traverse(netdev->dpif_type, flush_in_vport_cb, netdev);
+    }
 
     return 0;
 }
